//**************************************************************************//
//																			//
// File:		ISRs.c														//
//																			//
// Project:		Example Host Loader File									//
//																			//
// Description: This file contains the Interrupt Service Routines for the	//
//				Target DSP.													//
//																			//
//**************************************************************************//

#include "Example Host Loader File.h"

//--------------------------------------------------------------------------//
// Function:	Timer0_ISR													//
//																			//
// Parameters:	iSignal (not used):											//
//					Number of interrupt										//
//																			//
// Return:		None														//
//																			//
// Description:	This ISR toggles PF0 every time a Timer0 interrupt occurs.	//
//--------------------------------------------------------------------------//
void Timer0_ISR(int iSignal)
{
	// confirm interrupt handling
	sysreg_write(sysreg_IOPG, Timer_Page);
	io_space_write(GSR0, 0x0001);

	// toggle LED
	sysreg_write(sysreg_IOPG, General_Purpose_IO);
	if(io_space_read(FLAGS) & 0x0001) io_space_write(FLAGC, 0x0001);	// turn LED off
		else io_space_write(FLAGS, 0x0001);		// turn LED on
}

//--------------------------------------------------------------------------//
// Function:	HPI_ISR														//
//																			//
// Parameters:	iSignal (not used):											//
//					Number of interrupt										//
//																			//
// Return:		None														//
//																			//
// Description:	If this program (loader file) is transferred to the DSP		//
//				using Host Port DMA, and "interrupt on completion" is		//
//				enabled, the Host interrupt will be latched and this ISR	//
//				will be executed after the initialisation of interrupts.	//
//				It writes handshake values into defined memory locations	//
//				that can be checked by the Host to ensure that program		//
//				execution started successfully after booting. The ISR		//
//				disables itself, so it is executed only once.				//
//--------------------------------------------------------------------------//
void HPI_ISR(int iSignal)
{
	// pointer to handshake location in DM
	int *pHandshake_Address_DM = (int *)HANDSHAKE_ADDRESS_DM;

	// confirm interrupt handling
	sysreg_write(sysreg_IOPG, Host_Port_Bus_Interface_Page);
	io_space_write(HOSTD_IRQ, 0x0001);

	// disable Host interrupt
	interrupt(SIG_INT4, SIG_IGN);

	// write handshake value to DM
	*pHandshake_Address_DM = HANDSHAKE_VALUE_DM;

	// write handshake value to PM
	asm("i1 = HANDSHAKE_ADDRESS_PM;");			// inline assembler is necessary, because C
	asm("m1 = 0x0000;");						// doesn't provide means for accessing 24-bit PM
	asm("ax0 = HANDSHAKE_VALUE_PM_MSW;");
	asm("px = HANDSHAKE_VALUE_PM_LSW;");
	asm("pm(i1 + m1) = ax0;");
}
